// Personal includes
#include "USBRemoteBox.h"
#include "Helpers.h"

//----------------------------------------------------------------------//
//								Definitions								//
//----------------------------------------------------------------------//
#define DEBUG_MTL_INSTRUMENT_USBREMOTEBOX					1
#define DEBUG_MTL_INSTRUMENT_USBREMOTEBOX_ERRORS_ONLY		1
#if (defined(_DEBUG) && defined(DEBUG_MTL_INSTRUMENT_USBREMOTEBOX) && DEBUG_MTL_INSTRUMENT_USBREMOTEBOX)
#if (defined(DEBUG_MTL_INSTRUMENT_USBREMOTEBOX_ERRORS_ONLY) && DEBUG_MTL_INSTRUMENT_USBREMOTEBOX_ERRORS_ONLY)
#define MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(__X__)
#else
#define MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(__X__)		COUT(__X__)
#endif
#define MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_CERR(__X__)		CERR(__X__)
#else
#define MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(__X__)
#define MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_CERR(__X__)
#endif

// Utilities
#define UNUSED_VAR(x)	(void)x;	// Prevents from unereferenced variable when compiling in release

using namespace MTL::Instrument;

const std::string		CUSBRemoteBox::RSRC_FILTER = "USB[0-9]*::0x1BFA::0x1F6D::[0-9]+::RAW";

CUSBRemoteBox::CUSBRemoteBox(CVISAResourceManager & rRM, tResourceName Rsrc)
	: CVISAInstrument(rRM, Rsrc)
{

}

CUSBRemoteBox::~CUSBRemoteBox()
{
	Disconnect();
}

bool CUSBRemoteBox::Connect()
{
	MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(MTL__FUNCTION_NAME__ << std::endl);
	CLockGuard<CRecursiveMutex> l_LockGuard(m_Lock);

	try
	{
		if (!CVISAInstrument::Open(eOpenAccessMode::ExclusiveLock, m_Timeout_ms))
			throw false;
		if (!CVISAInstrument::SetTimeout(m_Timeout_ms))
			throw false;

		ViByte l_BufIn[1];
		ViUInt16 l_RetCount;
		if (!CVISAInstrument::UsbControlIn(USB_ENDPOINT_IN | USB_REQUEST_TYPE_VENDOR | USB_RECIPIENT_DEVICE, 255, 0x370B, 0, sizeof(l_BufIn), (ViPBuf)l_BufIn, l_RetCount))
			throw false;
		if (!CVISAInstrument::UsbControlOut(USB_ENDPOINT_OUT | USB_REQUEST_TYPE_VENDOR | USB_RECIPIENT_INTERFACE, 2, 0x0002, 0, 0, NULL))
			throw false;
	}
	catch (bool & rE)
	{
		UNUSED_VAR(rE);
		Disconnect();
		return false;
	}
	return true;
}

void CUSBRemoteBox::Disconnect()
{
	MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(MTL__FUNCTION_NAME__ << std::endl);
	CLockGuard<CRecursiveMutex> l_LockGuard(m_Lock);

	CVISAInstrument::Close();
}

bool CUSBRemoteBox::Update(const uLEDs LEDs, uButtons & rButtons)
{
	MTL_INSTRUMENT_USBREMOTEBOX_DEBUG_COUT(MTL__FUNCTION_NAME__ << std::endl);
	CLockGuard<CRecursiveMutex> l_LockGuard(m_Lock);

	try
	{
		if (!CVISAInstrument::Write((ViPBuf)&LEDs.Buf, sizeof(LEDs.Buf)))
			throw false;
		ViUInt32 l_RetLen;
		if (!CVISAInstrument::Read((ViPBuf)&rButtons.Buf, (ViUInt32)sizeof(rButtons.Buf), l_RetLen))
			throw false;
	}
	catch (bool & rE)
	{
		UNUSED_VAR(rE);
		return false;
	}
	return true;
}
