//////////////////////////////////////////////////////////////////////////
/// \file
/// \brief Platform Dependent Definitions.
// See https://blog.kowalczyk.info/article/j/guide-to-predefined-macros-in-c-compilers-gcc-clang-msvc-etc..html

#pragma once

//---------------------------------------------------------------------------//
//	Platform Dependant Definitions
//---------------------------------------------------------------------------//
#ifdef _WIN32

	// Definitions
	#define MTL__FUNCTION_NAME__			__FUNCTION__
	#define MTL__FUNCTION_SIGNATURE__		__FUNCSIG__

#endif
#if defined(__GNUC__) || defined(__APPLE__)

	// Definitions
	#define MTL__FUNCTION_NAME__			__FUNCTION__
	#define MTL__FUNCTION_SIGNATURE__		__PRETTY_FUNCTION__

#endif

//----------------------------------------------------------------------//
//							Basic Types									//
//----------------------------------------------------------------------//
#if defined(_WIN32) || defined(__GNUC__) || defined(__APPLE__)
	// According to C++ standard, platform / data models are as follows:
	//	ILP32: Win32 & Linux 32 bits & Mac OS X 32 bits
	//	LLP64: Win64
	//	LP64:  Linux 64 bits & Mac OS X
	// The following definitions remain valid for the three ILP32, LLP64 and LP64 data models
	typedef signed char				I8;		///< \brief signed byte.
	typedef signed short int		I16;	///< \brief 16-bit signed integer.
	typedef signed int				I32;	///< \brief 32-bit signed integer.
	typedef signed long long int	I64;	///< \brief 64-bit signed integer.
	typedef unsigned char			U8;		///< \brief unsigned byte.
	typedef unsigned short int		U16;	///< \brief 16-bit unsigned integer.
	typedef unsigned int			U32;	///< \brief 32-bit unsigned integer.
	typedef unsigned long long int	U64;	///< \brief 64-bit unsigned integer.
	typedef float					F32;	///< \brief 32-bit floating-point number.
	typedef double					F64;	///< \brief 64-bit floating-point number.
	
	// We rely on C++ 11 for IEEE754 Not A Number representations
	#define SPqNaN		std::numeric_limits<F32>::quiet_NaN()		///< \brief Single Precision Quiet NaN.
	#define SPsNaN		std::numeric_limits<F32>::signaling_NaN()	///< \brief Single Precision Signaling NaN.
	#define DPqNaN		std::numeric_limits<F64>::quiet_NaN()		///< \brief Double Precision Quiet NaN.
	#define DPsNaN		std::numeric_limits<F64>::signaling_NaN()	///< \brief Double Precision Signaling NaN.
#endif
